import { Inject, Injectable, InjectionToken, NgModule, PLATFORM_ID } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';
import { Router } from '@angular/router';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import { merge } from 'rxjs/observable/merge';
import { catchError, filter, mapTo, startWith } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

var OFFLINE_CONFIG_DEFAULT = {
    routeOffline: '/offline',
    routeUnavailable: '/unavailable',
    guardsRedirect: true
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var OFFLINE_CONFIG_ROUTE_OFFLINE = new InjectionToken('offline-config-route-offline');
var OFFLINE_CONFIG_ROUTE_UNAVAILABLE = new InjectionToken('offline-config-route-unavailable');
var OFFLINE_CONFIG_GUARDS_REDIRECT = new InjectionToken('offline-config-guards-redirect');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var Offline = (function () {
    function Offline(router, platformId, routeOffline, routeUnavailable, guardsRedirect) {
        if (routeOffline === void 0) { routeOffline = OFFLINE_CONFIG_DEFAULT.routeOffline; }
        if (routeUnavailable === void 0) { routeUnavailable = OFFLINE_CONFIG_DEFAULT.routeUnavailable; }
        if (guardsRedirect === void 0) { guardsRedirect = OFFLINE_CONFIG_DEFAULT.guardsRedirect; }
        this.router = router;
        this.platformId = platformId;
        this.routeOffline = routeOffline;
        this.routeUnavailable = routeUnavailable;
        this.guardsRedirect = guardsRedirect;
        /* Store instance in a static property to allow access to injected services in the RxJS static operator */
        Offline.instance = this;
        this.initConnectionObservable();
    }
    Object.defineProperty(Offline.prototype, "isOnline", {
        get: /**
         * @return {?}
         */
        function () {
            return isPlatformBrowser(this.platformId) ? navigator.onLine : true;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @template T
     * @return {?}
     */
    Offline.catch = /**
     * @template T
     * @return {?}
     */
    function () {
        return catchError(Offline.catchCallback);
    };
    /**
     * @template T
     * @param {?} error
     * @param {?} caught
     * @return {?}
     */
    Offline.catchCallback = /**
     * @template T
     * @param {?} error
     * @param {?} caught
     * @return {?}
     */
    function (error, caught) {
        if (!Offline.instance) {
            console.log('You need to import OfflineModule in your AppModule AND to inject the Offline service in your AppComponent constructor.');
            throw error;
        }
        else {
            var /** @type {?} */ cancel = caught.pipe(filter(function () { return false; }));
            if (!Offline.instance.isOnline) {
                Offline.instance.router.navigate([Offline.instance.routeOffline]);
                return cancel;
            }
            else if (error.status && (error.status >= 500 && error.status < 600)) {
                Offline.instance.router.navigate([Offline.instance.routeUnavailable]);
                return cancel;
            }
            else {
                throw error;
            }
        }
    };
    /**
     * @return {?}
     */
    Offline.prototype.canActivate = /**
     * @return {?}
     */
    function () {
        return this.guard();
    };
    /**
     * @return {?}
     */
    Offline.prototype.canActivateChild = /**
     * @return {?}
     */
    function () {
        return this.guard();
    };
    /**
     * @return {?}
     */
    Offline.prototype.canLoad = /**
     * @return {?}
     */
    function () {
        return this.guard();
    };
    /**
     * @return {?}
     */
    Offline.prototype.initConnectionObservable = /**
     * @return {?}
     */
    function () {
        this.connectionChanges = !isPlatformBrowser(this.platformId) ? of(true) : merge(startWith(this.isOnline), fromEvent(document, 'online').pipe(mapTo(true)), fromEvent(document, 'offline').pipe(mapTo(false)));
    };
    /**
     * @return {?}
     */
    Offline.prototype.guard = /**
     * @return {?}
     */
    function () {
        if (!this.isOnline) {
            if (this.guardsRedirect) {
                this.router.navigate([this.routeOffline]);
            }
            return false;
        }
        return true;
    };
    Offline.instance = null;
    Offline.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Offline.ctorParameters = function () { return [
        { type: Router, },
        { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] },] },
        { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_ROUTE_OFFLINE,] },] },
        { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_ROUTE_UNAVAILABLE,] },] },
        { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_GUARDS_REDIRECT,] },] },
    ]; };
    return Offline;
}());
var catchOffline = Offline.catch;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ɵ0 = OFFLINE_CONFIG_DEFAULT.routeOffline;
var ɵ1 = OFFLINE_CONFIG_DEFAULT.routeOffline;
var ɵ2 = OFFLINE_CONFIG_DEFAULT.guardsRedirect;
var OfflineModule = (function () {
    function OfflineModule() {
    }
    /**
     * @param {?=} userConfig
     * @return {?}
     */
    OfflineModule.forRoot = /**
     * @param {?=} userConfig
     * @return {?}
     */
    function (userConfig) {
        if (userConfig === void 0) { userConfig = {}; }
        return {
            ngModule: OfflineModule,
            providers: [
                { provide: OFFLINE_CONFIG_ROUTE_OFFLINE, useValue: userConfig.routeOffline },
                { provide: OFFLINE_CONFIG_ROUTE_UNAVAILABLE, useValue: userConfig.routeUnavailable },
                { provide: OFFLINE_CONFIG_GUARDS_REDIRECT, useValue: userConfig.guardsRedirect },
            ]
        };
    };
    OfflineModule.decorators = [
        { type: NgModule, args: [{
                    providers: [
                        { provide: OFFLINE_CONFIG_ROUTE_OFFLINE, useValue: ɵ0 },
                        { provide: OFFLINE_CONFIG_ROUTE_UNAVAILABLE, useValue: ɵ1 },
                        { provide: OFFLINE_CONFIG_GUARDS_REDIRECT, useValue: ɵ2 },
                        Offline,
                    ]
                },] },
    ];
    return OfflineModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { Offline, catchOffline, OfflineModule, OFFLINE_CONFIG_DEFAULT as ɵa, OFFLINE_CONFIG_GUARDS_REDIRECT as ɵd, OFFLINE_CONFIG_ROUTE_OFFLINE as ɵb, OFFLINE_CONFIG_ROUTE_UNAVAILABLE as ɵc };
//# sourceMappingURL=ngx-pwa-offline.es5.js.map
