import { Inject, Injectable, InjectionToken, NgModule, PLATFORM_ID } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';
import { Router } from '@angular/router';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import { merge } from 'rxjs/observable/merge';
import { catchError, filter, mapTo, startWith } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

const OFFLINE_CONFIG_DEFAULT = {
    routeOffline: '/offline',
    routeUnavailable: '/unavailable',
    guardsRedirect: true
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const OFFLINE_CONFIG_ROUTE_OFFLINE = new InjectionToken('offline-config-route-offline');
const OFFLINE_CONFIG_ROUTE_UNAVAILABLE = new InjectionToken('offline-config-route-unavailable');
const OFFLINE_CONFIG_GUARDS_REDIRECT = new InjectionToken('offline-config-guards-redirect');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Offline {
    /**
     * @param {?} router
     * @param {?} platformId
     * @param {?=} routeOffline
     * @param {?=} routeUnavailable
     * @param {?=} guardsRedirect
     */
    constructor(router, platformId, routeOffline = OFFLINE_CONFIG_DEFAULT.routeOffline, routeUnavailable = OFFLINE_CONFIG_DEFAULT.routeUnavailable, guardsRedirect = OFFLINE_CONFIG_DEFAULT.guardsRedirect) {
        this.router = router;
        this.platformId = platformId;
        this.routeOffline = routeOffline;
        this.routeUnavailable = routeUnavailable;
        this.guardsRedirect = guardsRedirect;
        /* Store instance in a static property to allow access to injected services in the RxJS static operator */
        Offline.instance = this;
        this.initConnectionObservable();
    }
    /**
     * @return {?}
     */
    get isOnline() {
        return isPlatformBrowser(this.platformId) ? navigator.onLine : true;
    }
    /**
     * @template T
     * @return {?}
     */
    static catch() {
        return catchError(Offline.catchCallback);
    }
    /**
     * @template T
     * @param {?} error
     * @param {?} caught
     * @return {?}
     */
    static catchCallback(error, caught) {
        if (!Offline.instance) {
            console.log('You need to import OfflineModule in your AppModule AND to inject the Offline service in your AppComponent constructor.');
            throw error;
        }
        else {
            const /** @type {?} */ cancel = caught.pipe(filter(() => false));
            if (!Offline.instance.isOnline) {
                Offline.instance.router.navigate([Offline.instance.routeOffline]);
                return cancel;
            }
            else if (error.status && (error.status >= 500 && error.status < 600)) {
                Offline.instance.router.navigate([Offline.instance.routeUnavailable]);
                return cancel;
            }
            else {
                throw error;
            }
        }
    }
    /**
     * @return {?}
     */
    canActivate() {
        return this.guard();
    }
    /**
     * @return {?}
     */
    canActivateChild() {
        return this.guard();
    }
    /**
     * @return {?}
     */
    canLoad() {
        return this.guard();
    }
    /**
     * @return {?}
     */
    initConnectionObservable() {
        this.connectionChanges = !isPlatformBrowser(this.platformId) ? of(true) : merge(startWith(this.isOnline), fromEvent(document, 'online').pipe(mapTo(true)), fromEvent(document, 'offline').pipe(mapTo(false)));
    }
    /**
     * @return {?}
     */
    guard() {
        if (!this.isOnline) {
            if (this.guardsRedirect) {
                this.router.navigate([this.routeOffline]);
            }
            return false;
        }
        return true;
    }
}
Offline.instance = null;
Offline.decorators = [
    { type: Injectable },
];
/** @nocollapse */
Offline.ctorParameters = () => [
    { type: Router, },
    { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_ROUTE_OFFLINE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_ROUTE_UNAVAILABLE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [OFFLINE_CONFIG_GUARDS_REDIRECT,] },] },
];
const catchOffline = Offline.catch;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ɵ0 = OFFLINE_CONFIG_DEFAULT.routeOffline;
const ɵ1 = OFFLINE_CONFIG_DEFAULT.routeOffline;
const ɵ2 = OFFLINE_CONFIG_DEFAULT.guardsRedirect;
class OfflineModule {
    /**
     * @param {?=} userConfig
     * @return {?}
     */
    static forRoot(userConfig = {}) {
        return {
            ngModule: OfflineModule,
            providers: [
                { provide: OFFLINE_CONFIG_ROUTE_OFFLINE, useValue: userConfig.routeOffline },
                { provide: OFFLINE_CONFIG_ROUTE_UNAVAILABLE, useValue: userConfig.routeUnavailable },
                { provide: OFFLINE_CONFIG_GUARDS_REDIRECT, useValue: userConfig.guardsRedirect },
            ]
        };
    }
}
OfflineModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    { provide: OFFLINE_CONFIG_ROUTE_OFFLINE, useValue: ɵ0 },
                    { provide: OFFLINE_CONFIG_ROUTE_UNAVAILABLE, useValue: ɵ1 },
                    { provide: OFFLINE_CONFIG_GUARDS_REDIRECT, useValue: ɵ2 },
                    Offline,
                ]
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { Offline, catchOffline, OfflineModule, OFFLINE_CONFIG_DEFAULT as ɵa, OFFLINE_CONFIG_GUARDS_REDIRECT as ɵd, OFFLINE_CONFIG_ROUTE_OFFLINE as ɵb, OFFLINE_CONFIG_ROUTE_UNAVAILABLE as ɵc };
//# sourceMappingURL=ngx-pwa-offline.js.map
